@@ -19,40 +19,67 @@
 package org.apache.felix.framework;
 
 import java.util.*;
-
-import org.apache.felix.framework.util.FelixConstants;
-import org.osgi.framework.*;
-import org.osgi.framework.hooks.service.*;
-import org.osgi.framework.launch.Framework;
+import org.apache.felix.framework.capabilityset.CapabilitySet;
+import org.apache.felix.framework.capabilityset.SimpleFilter;
+import org.apache.felix.framework.wiring.BundleCapabilityImpl;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.Constants;
+import org.osgi.framework.ServiceEvent;
+import org.osgi.framework.ServiceException;
+import org.osgi.framework.ServiceFactory;
+import org.osgi.framework.ServiceReference;
+import org.osgi.framework.ServiceRegistration;
+import org.osgi.framework.wiring.BundleCapability;
 
 public class ServiceRegistry
 {
     private final Logger m_logger;
     private long m_currentServiceId = 1L;
     // Maps bundle to an array of service registrations.
-    private final Map m_serviceRegsMap = Collections.synchronizedMap(new HashMap());
+    private final Map m_regsMap = Collections.synchronizedMap(new HashMap());
+    // Capability set for all service registrations.
+    private final CapabilitySet m_regCapSet;
+
     // Maps registration to thread to keep track when a
     // registration is in use, which will cause other
     // threads to wait.
-    private Map m_lockedRegsMap = new HashMap();
+    private final Map m_lockedRegsMap = new HashMap();
     // Maps bundle to an array of usage counts.
-    private Map m_inUseMap = new HashMap();
+    private final Map m_inUseMap = new HashMap();
 
     private final ServiceRegistryCallbacks m_callbacks;
 
-    private final Set m_eventHooks = new TreeSet(Collections.reverseOrder());
-    private final Set m_findHooks = new TreeSet(Collections.reverseOrder());
-    private final Set m_listenerHooks = new TreeSet(Collections.reverseOrder());
+    private final WeakHashMap<ServiceReference, ServiceReference> m_blackList =
+        new WeakHashMap<ServiceReference, ServiceReference>();
+
+    private final static Class<?>[] m_hookClasses = {
+        org.osgi.framework.hooks.bundle.FindHook.class,
+        org.osgi.framework.hooks.bundle.EventHook.class,
+        org.osgi.framework.hooks.service.EventHook.class,
+        org.osgi.framework.hooks.service.EventListenerHook.class,
+        org.osgi.framework.hooks.service.FindHook.class,
+        org.osgi.framework.hooks.service.ListenerHook.class,
+        org.osgi.framework.hooks.weaving.WeavingHook.class,
+        org.osgi.framework.hooks.resolver.ResolverHookFactory.class,
+        org.osgi.service.url.URLStreamHandlerService.class,
+        java.net.ContentHandler.class
+    };
+    private final Map<Class<?>, Set<ServiceReference<?>>> m_allHooks =
+        new HashMap<Class<?>, Set<ServiceReference<?>>>();
 
     public ServiceRegistry(Logger logger, ServiceRegistryCallbacks callbacks)
     {
         m_logger = logger;
         m_callbacks = callbacks;
+
+        List indices = new ArrayList();
+        indices.add(Constants.OBJECTCLASS);
+        m_regCapSet = new CapabilitySet(indices, false);
     }
 
     public ServiceReference[] getRegisteredServices(Bundle bundle)
     {
-        ServiceRegistration[] regs = (ServiceRegistration[]) m_serviceRegsMap.get(bundle);
+        ServiceRegistration[] regs = (ServiceRegistration[]) m_regsMap.get(bundle);
         if (regs != null)
         {
             List refs = new ArrayList(regs.length);
@@ -72,31 +99,27 @@ public class ServiceRegistry
         return null;
     }
 
+    // Caller is expected to fire REGISTERED event.
     public ServiceRegistration registerService(
         Bundle bundle, String[] classNames, Object svcObj, Dictionary dict)
     {
-        ServiceRegistration reg = null;
+        ServiceRegistrationImpl reg = null;
 
         synchronized (this)
         {
             // Create the service registration.
             reg = new ServiceRegistrationImpl(
                 this, bundle, classNames, new Long(m_currentServiceId++), svcObj, dict);
-                        
+
             // Keep track of registered hooks.
             addHooks(classNames, svcObj, reg.getReference());
 
             // Get the bundles current registered services.
-            ServiceRegistration[] regs = (ServiceRegistration[]) m_serviceRegsMap.get(bundle);
-            m_serviceRegsMap.put(bundle, addServiceRegistration(regs, reg));
+            ServiceRegistration[] regs = (ServiceRegistration[]) m_regsMap.get(bundle);
+            m_regsMap.put(bundle, addServiceRegistration(regs, reg));
+            m_regCapSet.addCapability((BundleCapabilityImpl) reg.getReference());
         }
 
-        // Notify callback objects about registered service.
-        if (m_callbacks != null)
-        {
-            m_callbacks.serviceChanged(
-                new ServiceEvent(ServiceEvent.REGISTERED, reg.getReference()), null);
-        }
         return reg;
     }
 
@@ -114,8 +137,9 @@ public class ServiceRegistry
             // new bundles will be able to look up the service.
 
             // Now remove the registered service.
-            ServiceRegistration[] regs = (ServiceRegistration[]) m_serviceRegsMap.get(bundle);
-            m_serviceRegsMap.put(bundle, removeServiceRegistration(regs, reg));
+            ServiceRegistration[] regs = (ServiceRegistration[]) m_regsMap.get(bundle);
+            m_regsMap.put(bundle, removeServiceRegistration(regs, reg));
+            m_regCapSet.removeCapability((BundleCapabilityImpl) reg.getReference());
         }
 
         // Notify callback objects about unregistering service.
@@ -151,7 +175,7 @@ public class ServiceRegistry
         ServiceRegistration[] regs = null;
         synchronized (this)
         {
-            regs = (ServiceRegistration[]) m_serviceRegsMap.get(bundle);
+            regs = (ServiceRegistration[]) m_regsMap.get(bundle);
         }
 
         // Note, there is no race condition here with respect to the
@@ -171,73 +195,35 @@ public class ServiceRegistry
         // Now remove the bundle itself.
         synchronized (this)
         {
-            m_serviceRegsMap.remove(bundle);
+            m_regsMap.remove(bundle);
         }
     }
 
-    public List getServiceReferences(String className, Filter filter)
+    public synchronized List getServiceReferences(String className, SimpleFilter filter)
     {
-        // Create a filtered list of service references.
-        List list = new ArrayList();
-
-        Object[] registrations = m_serviceRegsMap.values().toArray();
-        
-        // Iterator over all service registrations.
-        for (int i = 0; i < registrations.length; i++)
+        if ((className == null) && (filter == null))
         {
-            ServiceRegistration[] regs = (ServiceRegistration[]) registrations[i];
-
-            for (int regIdx = 0;
-                (regs != null) && (regIdx < regs.length);
-                regIdx++)
-            {
-                try
-                {
-                    // Determine if the registered services matches
-                    // the search criteria.
-                    boolean matched = false;
-
-                    // If className is null, then look at filter only.
-                    if ((className == null) &&
-                        ((filter == null) || filter.match(regs[regIdx].getReference())))
-                    {
-                        matched = true;
-                    }
-                    // If className is not null, then first match the
-                    // objectClass property before looking at the
-                    // filter.
-                    else if (className != null)
-                    {
-                        String[] objectClass = (String[])
-                            ((ServiceRegistrationImpl) regs[regIdx])
-                                .getProperty(FelixConstants.OBJECTCLASS);
-                        for (int classIdx = 0;
-                            classIdx < objectClass.length;
-                            classIdx++)
-                        {
-                            if (objectClass[classIdx].equals(className) &&
-                                ((filter == null) || filter.match(regs[regIdx].getReference())))
-                            {
-                                matched = true;
-                                break;
-                            }
-                        }
-                    }
-
-                    // Add reference if it was a match.
-                    if (matched)
-                    {
-                        list.add(regs[regIdx].getReference());
-                    }
-                }
-                catch (IllegalStateException ex)
-                {
-                    // Don't include the reference as it is not valid anymore
-                }
-            }
+            // Return all services.
+            filter = new SimpleFilter(Constants.OBJECTCLASS, "*", SimpleFilter.PRESENT);
         }
+        else if ((className != null) && (filter == null))
+        {
+            // Return services matching the class name.
+            filter = new SimpleFilter(Constants.OBJECTCLASS, className, SimpleFilter.EQ);
+        }
+        else if ((className != null) && (filter != null))
+        {
+            // Return services matching the class name and filter.
+            List filters = new ArrayList(2);
+            filters.add(new SimpleFilter(Constants.OBJECTCLASS, className, SimpleFilter.EQ));
+            filters.add(filter);
+            filter = new SimpleFilter(null, filters, SimpleFilter.AND);
+        }
+        // else just use the specified filter.
 
-        return list;
+        Set<BundleCapability> matches = m_regCapSet.match(filter, false);
+
+        return new ArrayList(matches);
     }
 
     public synchronized ServiceReference[] getServicesInUse(Bundle bundle)
@@ -255,7 +241,7 @@ public class ServiceRegistry
         return null;
     }
 
-    public Object getService(Bundle bundle, ServiceReference ref)
+    public <S> S getService(Bundle bundle, ServiceReference<S> ref)
     {
         UsageCount usage = null;
         Object svcObj = null;
@@ -273,8 +259,10 @@ public class ServiceRegistry
                 // We don't allow cycles when we call out to the service factory.
                 if (o.equals(Thread.currentThread()))
                 {
-                    throw new IllegalStateException(
-                        "ServiceFactory.getService() resulted in a cycle.");
+                    throw new ServiceException(
+                        "ServiceFactory.getService() resulted in a cycle.",
+                        ServiceException.FACTORY_ERROR,
+                        null);
                 }
 
                 // Otherwise, wait for it to be freed.
@@ -346,7 +334,7 @@ public class ServiceRegistry
             }
         }
 
-        return svcObj;
+        return (S) svcObj;
     }
 
     public boolean ungetService(Bundle bundle, ServiceReference ref)
@@ -469,7 +457,7 @@ public class ServiceRegistry
     }
 
     public synchronized Bundle[] getUsingBundles(ServiceReference ref)
-    {        
+    {
         Bundle[] bundles = null;
         for (Iterator iter = m_inUseMap.entrySet().iterator(); iter.hasNext(); )
         {
@@ -661,46 +649,42 @@ public class ServiceRegistry
         }
     }
 
-    private void addHooks(String[] classNames, Object svcObj, ServiceReference ref)
+    //
+    // Hook-related methods.
+    //
+
+    boolean isHookBlackListed(ServiceReference sr)
     {
-        if (isHook(classNames, EventHook.class, svcObj))
-        {
-            synchronized (m_eventHooks)
-            {
-                m_eventHooks.add(ref);
-            }
-        }
-
-        if (isHook(classNames, FindHook.class, svcObj))
-        {
-            synchronized (m_findHooks)
-            {
-                m_findHooks.add(ref);
-            }
-        }
-
-        if (isHook(classNames, ListenerHook.class, svcObj))
-        {
-            synchronized (m_listenerHooks)
-            {
-                m_listenerHooks.add(ref);
-            }
-        }
+        return m_blackList.containsKey(sr);
     }
-    
-    static boolean isHook(String[] classNames, Class hookClass, Object svcObj)
+
+    void blackListHook(ServiceReference sr)
     {
-        if (svcObj instanceof ServiceFactory) 
+        m_blackList.put(sr, sr);
+    }
+
+    static boolean isHook(String[] classNames, Class<?> hookClass, Object svcObj)
+    {
+        // For a service factory, we can only match names.
+        if (svcObj instanceof ServiceFactory)
         {
-            return Arrays.asList(classNames).contains(hookClass.getName());
+            for (String className : classNames)
+            {
+                if (className.equals(hookClass.getName()))
+                {
+                    return true;
+                }
+            }
         }
-        
+
+        // For a service object, check if its class matches.
         if (hookClass.isAssignableFrom(svcObj.getClass()))
         {
+            // But still only if it is registered under that interface.
             String hookName = hookClass.getName();
-            for (int i = 0; i < classNames.length; i++)
+            for (String className : classNames)
             {
-                if (classNames[i].equals(hookName))
+                if (className.equals(hookName))
                 {
                     return true;
                 }
@@ -709,89 +693,73 @@ public class ServiceRegistry
         return false;
     }
 
+    private void addHooks(String[] classNames, Object svcObj, ServiceReference<?> ref)
+    {
+        for (Class<?> hookClass : m_hookClasses)
+        {
+            if (isHook(classNames, hookClass, svcObj))
+            {
+                synchronized (m_allHooks)
+                {
+                    Set<ServiceReference<?>> hooks = m_allHooks.get(hookClass);
+                    if (hooks == null)
+                    {
+                        hooks = new HashSet<ServiceReference<?>>();
+                        m_allHooks.put(hookClass, hooks);
+                    }
+                    hooks.add(ref);
+                }
+            }
+        }
+    }
+
     private void removeHook(ServiceReference ref)
     {
         Object svcObj = ((ServiceRegistrationImpl.ServiceReferenceImpl) ref)
             .getRegistration().getService();
         String [] classNames = (String[]) ref.getProperty(Constants.OBJECTCLASS);
-        
-        if (isHook(classNames, EventHook.class, svcObj)) 
-        {
-            synchronized (m_eventHooks) 
-            {
-                m_eventHooks.remove(ref);
-            }
-        }
-        
-        if (isHook(classNames, FindHook.class, svcObj)) 
-        {
-            synchronized (m_findHooks)
-            {
-                m_findHooks.remove(ref);
-            }
-        }
 
-        if (isHook(classNames, ListenerHook.class, svcObj))
+        for (Class<?> hookClass : m_hookClasses)
         {
-            synchronized (m_listenerHooks)
+            if (isHook(classNames, hookClass, svcObj))
             {
-                m_listenerHooks.remove(ref);
+                synchronized (m_allHooks)
+                {
+                    Set<ServiceReference<?>> hooks = m_allHooks.get(hookClass);
+                    if (hooks != null)
+                    {
+                        hooks.remove(ref);
+                        if (hooks.isEmpty())
+                        {
+                            m_allHooks.remove(hookClass);
+                        }
+                    }
+                }
             }
         }
     }
 
-    public List getEventHooks()
+    public <S> Set<ServiceReference<S>> getHooks(Class<S> hookClass)
     {
-        synchronized (m_eventHooks)
+        synchronized (m_allHooks)
         {
-            return new ArrayList(m_eventHooks);
+            Set<ServiceReference<?>> hooks = m_allHooks.get(hookClass);
+            if (hooks != null)
+            {
+                SortedSet sorted = new TreeSet<ServiceReference<?>>(Collections.reverseOrder());
+                sorted.addAll(hooks);
+                return asTypedSortedSet(sorted);
+            }
+            return Collections.EMPTY_SET;
         }
     }
 
-    List getFindHooks()
+    private static <S> SortedSet<ServiceReference<S>> asTypedSortedSet(
+        SortedSet<ServiceReference<?>> ss)
     {
-        synchronized (m_findHooks)
-        {
-            return new ArrayList(m_findHooks);
-        }
+        return (SortedSet<ServiceReference<S>>) (SortedSet) ss;
     }
 
-    List getListenerHooks()
-    {
-        synchronized (m_listenerHooks)
-        {
-            return new ArrayList(m_listenerHooks);
-        }
-    }
-    
-    /**
-     * Invokes a Service Registry Hook
-     * @param ref The ServiceReference associated with the hook to be invoked, the hook
-     *        service object will be obtained through this object.
-     * @param framework The framework that is invoking the hook, typically the Felix object.
-     * @param callback This is a callback object that is invoked with the actual hook object to 
-     *        be used, either the plain hook, or one obtained from the ServiceFactory.
-     */
-    public void invokeHook(
-        ServiceReference ref, Framework framework, InvokeHookCallback callback)
-    {
-        Object hook = getService(framework, ref);
-        
-        try 
-        {
-            callback.invokeHook(hook);
-        }
-        catch (Throwable th)
-        {
-            m_logger.log(ref, Logger.LOG_WARNING,
-                "Problem invoking Service Registry Hook", th);
-        }
-        finally 
-        {
-            ungetService(framework, ref);
-        }
-    }    
-    
     private static class UsageCount
     {
         public int m_count = 0;
