@@ -328,7 +328,9 @@ public class URLHandlersStreamHandlerProxy extends URLStreamHandler
             } 
             catch (NoSuchMethodException e) 
             {
-                throw new UnsupportedOperationException(e);
+                RuntimeException rte = new UnsupportedOperationException(e.getMessage());
+                rte.initCause(e);
+                throw rte;
             }
             try 
             {
@@ -464,7 +466,7 @@ public class URLHandlersStreamHandlerProxy extends URLStreamHandler
     public void setURL(
         URL url, String protocol, String host, int port, String file, String ref)
     {
-        super.setURL(url, protocol, host, port, null, null, file, null, ref);
+        super.setURL(url, protocol, host, port, file, ref);
     }
 
     protected String toExternalForm(URL url)
