@@ -22,11 +22,23 @@ import java.io.*;
 import java.lang.reflect.*;
 import java.net.*;
 import java.security.*;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.Hashtable;
+import java.util.Map;
+import java.util.zip.ZipFile;
+import org.osgi.framework.Bundle;
 
 import org.osgi.framework.BundleActivator;
 import org.osgi.framework.BundleContext;
+import org.osgi.framework.BundleEvent;
+import org.osgi.framework.ServiceEvent;
+import org.osgi.framework.ServiceReference;
+import org.osgi.framework.hooks.resolver.ResolverHook;
+import org.osgi.framework.hooks.service.ListenerHook;
+import org.osgi.framework.wiring.BundleCapability;
+import org.osgi.framework.wiring.BundleRequirement;
+import org.osgi.framework.wiring.BundleRevision;
 
 /**
  * <p>
@@ -61,7 +73,7 @@ public class SecureAction
             try
             {
                 Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.INITIALIZE_CONTEXT, null);
+                actions.set(Actions.INITIALIZE_CONTEXT_ACTION, null);
                 m_acc = (AccessControlContext) AccessController.doPrivileged(actions);
             }
             catch (PrivilegedActionException ex)
@@ -137,7 +149,7 @@ public class SecureAction
             return ClassLoader.getSystemClassLoader();
         }
     }
-    
+
     public ClassLoader getClassLoader(Class clazz)
     {
         if (System.getSecurityManager() != null)
@@ -412,7 +424,7 @@ public class SecureAction
         }
     }
 
-    public InputStream getFileInputStream(File file) throws IOException
+    public FileInputStream getFileInputStream(File file) throws IOException
     {
         if (System.getSecurityManager() != null)
         {
@@ -420,7 +432,7 @@ public class SecureAction
             {
                 Actions actions = (Actions) m_actions.get();
                 actions.set(Actions.GET_FILE_INPUT_ACTION, file);
-                return (InputStream) AccessController.doPrivileged(actions, m_acc);
+                return (FileInputStream) AccessController.doPrivileged(actions, m_acc);
             }
             catch (PrivilegedActionException ex)
             {
@@ -437,7 +449,7 @@ public class SecureAction
         }
     }
 
-    public OutputStream getFileOutputStream(File file) throws IOException
+    public FileOutputStream getFileOutputStream(File file) throws IOException
     {
         if (System.getSecurityManager() != null)
         {
@@ -445,7 +457,7 @@ public class SecureAction
             {
                 Actions actions = (Actions) m_actions.get();
                 actions.set(Actions.GET_FILE_OUTPUT_ACTION, file);
-                return (OutputStream) AccessController.doPrivileged(actions, m_acc);
+                return (FileOutputStream) AccessController.doPrivileged(actions, m_acc);
             }
             catch (PrivilegedActionException ex)
             {
@@ -462,6 +474,27 @@ public class SecureAction
         }
     }
 
+    public URI toURI(File file)
+    {
+        if (System.getSecurityManager() != null)
+        {
+            try
+            {
+                Actions actions = (Actions) m_actions.get();
+                actions.set(Actions.TO_URI_ACTION, file);
+                return (URI) AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException ex)
+            {
+                throw (RuntimeException) ex.getException();
+            }
+        }
+        else
+        {
+            return file.toURI();
+        }
+    }
+
     public InputStream getURLConnectionInputStream(URLConnection conn)
         throws IOException
     {
@@ -562,15 +595,15 @@ public class SecureAction
         }
     }
 
-    public JarFileX openJAR(File file) throws IOException
+    public ZipFile openZipFile(File file) throws IOException
     {
         if (System.getSecurityManager() != null)
         {
             try
             {
                 Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.OPEN_JARX_ACTION, file);
-                return (JarFileX) AccessController.doPrivileged(actions, m_acc);
+                actions.set(Actions.OPEN_ZIPFILE_ACTION, file);
+                return (ZipFile) AccessController.doPrivileged(actions, m_acc);
             }
             catch (PrivilegedActionException ex)
             {
@@ -583,63 +616,10 @@ public class SecureAction
         }
         else
         {
-            return new JarFileX(file);
+            return new ZipFile(file);
         }
     }
 
-    public JarFileX openJAR(File file, boolean verify) throws IOException
-    {
-        if (System.getSecurityManager() != null)
-        {
-            try
-            {
-                Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.OPEN_JARX_VERIFY_ACTION, file, (verify ? Boolean.TRUE : Boolean.FALSE));
-                return (JarFileX) AccessController.doPrivileged(actions, m_acc);
-            }
-            catch (PrivilegedActionException ex)
-            {
-                if (ex.getException() instanceof IOException)
-                {
-                    throw (IOException) ex.getException();
-                }
-                throw (RuntimeException) ex.getException();
-            }
-        }
-        else
-        {
-            return new JarFileX(file, verify);
-        }
-    }
-// TODO: REFACTOR - SecureAction fix needed.
-/*
-    public ModuleClassLoader createModuleClassLoader(ModuleImpl impl)
-    {
-        return createModuleClassLoader(impl, null);
-    }
-
-    public ModuleClassLoader createModuleClassLoader(ModuleImpl impl,
-        ProtectionDomain protectionDomain)
-    {
-        if (System.getSecurityManager() != null)
-        {
-            try
-            {
-                Actions actions = (Actions) m_actions.get();
-                actions.set(Actions.CREATE_MODULECLASSLOADER_ACTION, impl, protectionDomain);
-                return (ModuleClassLoader) AccessController.doPrivileged(actions, m_acc);
-            }
-            catch (PrivilegedActionException ex)
-            {
-                throw (RuntimeException) ex.getException();
-            }
-        }
-        else
-        {
-            return new ModuleClassLoader(impl, protectionDomain);
-        }
-    }
-*/
     public void startActivator(BundleActivator activator, BundleContext context)
         throws Exception
     {
@@ -710,7 +690,7 @@ public class SecureAction
         if (System.getSecurityManager() != null)
         {
             Actions actions = (Actions) m_actions.get();
-            actions.set(Actions.ADD_EXTENSION_URL, extension, loader);
+            actions.set(Actions.ADD_EXTENSION_URL_ACTION, extension, loader);
             try
             {
                 AccessController.doPrivileged(actions, m_acc);
@@ -1041,7 +1021,7 @@ public class SecureAction
 
     private static void _flush(Class targetClazz, Object lock) throws Exception
     {
-        synchronized (lock) 
+        synchronized (lock)
         {
             Field[] fields = targetClazz.getDeclaredFields();
             // reset cache
@@ -1072,50 +1052,378 @@ public class SecureAction
         }
     }
 
+    public void invokeBundleFindHook(
+        org.osgi.framework.hooks.bundle.FindHook fh,
+        BundleContext bc, Collection<Bundle> bundles)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_BUNDLE_FIND_HOOK, fh, bc, bundles);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            fh.find(bc, bundles);
+        }
+    }
+
+    public void invokeBundleEventHook(
+        org.osgi.framework.hooks.bundle.EventHook eh,
+        BundleEvent event, Collection<BundleContext> contexts)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_BUNDLE_EVENT_HOOK, eh, contexts);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            eh.event(event, contexts);
+        }
+    }
+
+    public void invokeWeavingHook(
+        org.osgi.framework.hooks.weaving.WeavingHook wh,
+        org.osgi.framework.hooks.weaving.WovenClass wc)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_WEAVING_HOOK, wh, wc);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            wh.weave(wc);
+        }
+    }
+
+    public void invokeServiceEventHook(
+        org.osgi.framework.hooks.service.EventHook eh,
+        ServiceEvent event, Collection<BundleContext> contexts)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_SERVICE_EVENT_HOOK, eh, contexts);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            eh.event(event, contexts);
+        }
+    }
+
+    public void invokeServiceFindHook(
+        org.osgi.framework.hooks.service.FindHook fh,
+        BundleContext context, String name, String filter,
+        boolean allServices, Collection<ServiceReference<?>> references)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(
+                Actions.INVOKE_SERVICE_EVENT_HOOK, fh, context, name, filter,
+                (allServices) ? Boolean.TRUE : Boolean.FALSE, references);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            fh.find(context, name, filter, allServices, references);
+        }
+    }
+
+    public void invokeServiceListenerHookAdded(
+        org.osgi.framework.hooks.service.ListenerHook lh,
+        Collection<ListenerHook.ListenerInfo> listeners)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_SERVICE_LISTENER_HOOK_ADDED, lh, listeners);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            lh.added(listeners);
+        }
+    }
+
+    public void invokeServiceListenerHookRemoved(
+        org.osgi.framework.hooks.service.ListenerHook lh,
+        Collection<ListenerHook.ListenerInfo> listeners)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_SERVICE_LISTENER_HOOK_REMOVED, lh, listeners);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            lh.removed(listeners);
+        }
+    }
+
+    public void invokeServiceEventListenerHook(
+        org.osgi.framework.hooks.service.EventListenerHook elh,
+        ServiceEvent event,
+        Map<BundleContext, Collection<ListenerHook.ListenerInfo>> listeners)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_SERVICE_EVENT_LISTENER_HOOK, elh, listeners);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            elh.event(event, listeners);
+        }
+    }
+
+    public ResolverHook invokeResolverHookFactory(
+        org.osgi.framework.hooks.resolver.ResolverHookFactory rhf,
+        Collection<BundleRevision> triggers)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_RESOLVER_HOOK_FACTORY, rhf, triggers);
+            try
+            {
+                return (ResolverHook) AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            return rhf.begin(triggers);
+        }
+    }
+
+    public void invokeResolverHookResolvable(
+        org.osgi.framework.hooks.resolver.ResolverHook rh,
+        Collection<BundleRevision> candidates)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_RESOLVER_HOOK_RESOLVABLE, rh, candidates);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            rh.filterResolvable(candidates);
+        }
+    }
+
+    public void invokeResolverHookSingleton(
+        org.osgi.framework.hooks.resolver.ResolverHook rh,
+        BundleCapability singleton,
+        Collection<BundleCapability> collisions)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_RESOLVER_HOOK_SINGLETON, rh, singleton, collisions);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            rh.filterSingletonCollisions(singleton, collisions);
+        }
+    }
+
+    public void invokeResolverHookMatches(
+        org.osgi.framework.hooks.resolver.ResolverHook rh,
+        BundleRequirement req,
+        Collection<BundleCapability> candidates)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_RESOLVER_HOOK_MATCHES, rh, req, candidates);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            rh.filterMatches(req, candidates);
+        }
+    }
+
+    public void invokeResolverHookEnd(
+        org.osgi.framework.hooks.resolver.ResolverHook rh)
+        throws Exception
+    {
+        if (System.getSecurityManager() != null)
+        {
+            Actions actions = (Actions) m_actions.get();
+            actions.set(Actions.INVOKE_RESOLVER_HOOK_END, rh);
+            try
+            {
+                AccessController.doPrivileged(actions, m_acc);
+            }
+            catch (PrivilegedActionException e)
+            {
+                throw e.getException();
+            }
+        }
+        else
+        {
+            rh.end();
+        }
+    }
+
     private static class Actions implements PrivilegedExceptionAction
     {
-        public static final int INITIALIZE_CONTEXT = 0;
-
-        public static final int ADD_EXTENSION_URL = 1;
-        public static final int CREATE_MODULECLASSLOADER_ACTION = 2;
-        public static final int CREATE_TMPFILE_ACTION = 3;
-        public static final int CREATE_URL_ACTION = 4;
-        public static final int CREATE_URL_WITH_CONTEXT_ACTION = 5;
-        public static final int DELETE_FILE_ACTION = 6;
-        public static final int EXEC_ACTION = 7;
-        public static final int FILE_EXISTS_ACTION = 8;
-        public static final int FILE_IS_DIRECTORY_ACTION = 9;
-        public static final int FOR_NAME_ACTION = 10;
-        public static final int GET_ABSOLUTE_PATH_ACTION = 11;
-        public static final int GET_CONSTRUCTOR_ACTION = 12;
-        public static final int GET_DECLARED_CONSTRUCTOR_ACTION = 13;
-        public static final int GET_DECLARED_METHOD_ACTION = 14;
-        public static final int GET_FIELD_ACTION = 15;
-        public static final int GET_FILE_INPUT_ACTION = 16;
-        public static final int GET_FILE_OUTPUT_ACTION = 17;
-        public static final int GET_METHOD_ACTION = 19;
-        public static final int GET_POLICY_ACTION = 20;
-        public static final int GET_PROPERTY_ACTION = 21;
-        public static final int GET_PARENT_CLASS_LOADER_ACTION = 22;
-        public static final int GET_SYSTEM_CLASS_LOADER_ACTION = 23;
-        public static final int GET_URL_INPUT_ACTION = 24;
-        public static final int INVOKE_CONSTRUCTOR_ACTION = 25;
-        public static final int INVOKE_DIRECTMETHOD_ACTION = 26;
-        public static final int INVOKE_METHOD_ACTION = 27;
-        public static final int LIST_DIRECTORY_ACTION = 28;
-        public static final int MAKE_DIRECTORIES_ACTION = 29;
-        public static final int MAKE_DIRECTORY_ACTION = 30;
-        public static final int OPEN_JARX_ACTION = 31;
-        public static final int OPEN_JARX_VERIFY_ACTION = 32;
-        public static final int OPEN_URLCONNECTION_ACTION = 33;
-        public static final int RENAME_FILE_ACTION = 34;
-        public static final int SET_ACCESSIBLE_ACTION = 35;
-        public static final int START_ACTIVATOR_ACTION = 36;
-        public static final int STOP_ACTIVATOR_ACTION = 37;
-        public static final int SWAP_FIELD_ACTION = 38;
-        public static final int SYSTEM_EXIT_ACTION = 39;
-        public static final int FLUSH_FIELD_ACTION = 40;
-        public static final int GET_CLASS_LOADER_ACTION = 41;
+        public static final int INITIALIZE_CONTEXT_ACTION = 0;
+        public static final int ADD_EXTENSION_URL_ACTION = 1;
+        public static final int CREATE_TMPFILE_ACTION = 2;
+        public static final int CREATE_URL_ACTION = 3;
+        public static final int CREATE_URL_WITH_CONTEXT_ACTION = 4;
+        public static final int DELETE_FILE_ACTION = 5;
+        public static final int EXEC_ACTION = 6;
+        public static final int FILE_EXISTS_ACTION = 7;
+        public static final int FILE_IS_DIRECTORY_ACTION = 8;
+        public static final int FOR_NAME_ACTION = 9;
+        public static final int GET_ABSOLUTE_PATH_ACTION = 10;
+        public static final int GET_CONSTRUCTOR_ACTION = 11;
+        public static final int GET_DECLARED_CONSTRUCTOR_ACTION = 12;
+        public static final int GET_DECLARED_METHOD_ACTION = 13;
+        public static final int GET_FIELD_ACTION = 14;
+        public static final int GET_FILE_INPUT_ACTION = 15;
+        public static final int GET_FILE_OUTPUT_ACTION = 16;
+        public static final int TO_URI_ACTION = 17;
+        public static final int GET_METHOD_ACTION = 18;
+        public static final int GET_POLICY_ACTION = 19;
+        public static final int GET_PROPERTY_ACTION = 20;
+        public static final int GET_PARENT_CLASS_LOADER_ACTION = 21;
+        public static final int GET_SYSTEM_CLASS_LOADER_ACTION = 22;
+        public static final int GET_URL_INPUT_ACTION = 23;
+        public static final int INVOKE_CONSTRUCTOR_ACTION = 24;
+        public static final int INVOKE_DIRECTMETHOD_ACTION = 25;
+        public static final int INVOKE_METHOD_ACTION = 26;
+        public static final int LIST_DIRECTORY_ACTION = 27;
+        public static final int MAKE_DIRECTORIES_ACTION = 28;
+        public static final int MAKE_DIRECTORY_ACTION = 29;
+        public static final int OPEN_ZIPFILE_ACTION = 30;
+        public static final int OPEN_URLCONNECTION_ACTION = 31;
+        public static final int RENAME_FILE_ACTION = 32;
+        public static final int SET_ACCESSIBLE_ACTION = 33;
+        public static final int START_ACTIVATOR_ACTION = 34;
+        public static final int STOP_ACTIVATOR_ACTION = 35;
+        public static final int SWAP_FIELD_ACTION = 36;
+        public static final int SYSTEM_EXIT_ACTION = 37;
+        public static final int FLUSH_FIELD_ACTION = 38;
+        public static final int GET_CLASS_LOADER_ACTION = 39;
+        public static final int INVOKE_BUNDLE_FIND_HOOK = 40;
+        public static final int INVOKE_BUNDLE_EVENT_HOOK = 41;
+        public static final int INVOKE_WEAVING_HOOK = 42;
+        public static final int INVOKE_SERVICE_EVENT_HOOK = 43;
+        public static final int INVOKE_SERVICE_FIND_HOOK = 44;
+        public static final int INVOKE_SERVICE_LISTENER_HOOK_ADDED = 45;
+        public static final int INVOKE_SERVICE_LISTENER_HOOK_REMOVED = 46;
+        public static final int INVOKE_SERVICE_EVENT_LISTENER_HOOK = 47;
+        public static final int INVOKE_RESOLVER_HOOK_FACTORY = 48;
+        public static final int INVOKE_RESOLVER_HOOK_RESOLVABLE = 49;
+        public static final int INVOKE_RESOLVER_HOOK_SINGLETON = 50;
+        public static final int INVOKE_RESOLVER_HOOK_MATCHES = 51;
+        public static final int INVOKE_RESOLVER_HOOK_END = 52;
 
         private int m_action = -1;
         private Object m_arg1 = null;
@@ -1123,6 +1431,7 @@ public class SecureAction
         private Object m_arg3 = null;
         private Object m_arg4 = null;
         private Object m_arg5 = null;
+        private Object m_arg6 = null;
 
         public void set(int action)
         {
@@ -1171,6 +1480,18 @@ public class SecureAction
             m_arg5 = arg5;
         }
 
+        public void set(int action, Object arg1, Object arg2, Object arg3,
+            Object arg4, Object arg5, Object arg6)
+        {
+            m_action = action;
+            m_arg1 = arg1;
+            m_arg2 = arg2;
+            m_arg3 = arg3;
+            m_arg4 = arg4;
+            m_arg5 = arg5;
+            m_arg6 = arg6;
+        }
+
         private void unset()
         {
             m_action = -1;
@@ -1179,6 +1500,7 @@ public class SecureAction
             m_arg3 = null;
             m_arg4 = null;
             m_arg5 = null;
+            m_arg6 = null;
         }
 
         public Object run() throws Exception
@@ -1189,182 +1511,163 @@ public class SecureAction
             Object arg3 = m_arg3;
             Object arg4 = m_arg4;
             Object arg5 = m_arg5;
+            Object arg6 = m_arg6;
 
             unset();
 
-            if (action == INITIALIZE_CONTEXT)
+            switch (action)
             {
-                return AccessController.getContext();
-            }
-            else if (action == GET_PROPERTY_ACTION)
-            {
-                return System.getProperty((String) arg1, (String) arg2);
-            }
-            else if (action == GET_PARENT_CLASS_LOADER_ACTION)
-            {
-                return ((ClassLoader) arg1).getParent();
-            }
-            else if (action == GET_SYSTEM_CLASS_LOADER_ACTION)
-            {
-                return ClassLoader.getSystemClassLoader();
-            }
-            else if (action == FOR_NAME_ACTION)
-            {
-                return Class.forName((String) arg1);
-            }
-            else if (action == CREATE_URL_ACTION)
-            {
-                return new URL((String) arg1, (String) arg2,
-                    ((Integer) arg3).intValue(), (String) arg4,
-                    (URLStreamHandler) arg5);
-            }
-            else if (action == CREATE_URL_WITH_CONTEXT_ACTION)
-            {
-                return new URL((URL) arg1, (String) arg2,
-                    (URLStreamHandler) arg3);
-            }
-            else if (action == EXEC_ACTION)
-            {
-                return Runtime.getRuntime().exec((String) arg1);
-            }
-            else if (action == GET_ABSOLUTE_PATH_ACTION)
-            {
-                return ((File) arg1).getAbsolutePath();
-            }
-            else if (action == FILE_EXISTS_ACTION)
-            {
-                return ((File) arg1).exists() ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == FILE_IS_DIRECTORY_ACTION)
-            {
-                return ((File) arg1).isDirectory() ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == MAKE_DIRECTORY_ACTION)
-            {
-                return ((File) arg1).mkdir() ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == MAKE_DIRECTORIES_ACTION)
-            {
-                return ((File) arg1).mkdirs() ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == LIST_DIRECTORY_ACTION)
-            {
-                return ((File) arg1).listFiles();
-            }
-            else if (action == RENAME_FILE_ACTION)
-            {
-                return ((File) arg1).renameTo((File) arg2) ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == GET_FILE_INPUT_ACTION)
-            {
-                return new FileInputStream((File) arg1);
-            }
-            else if (action == GET_FILE_OUTPUT_ACTION)
-            {
-                return new FileOutputStream((File) arg1);
-            }
-            else if (action == DELETE_FILE_ACTION)
-            {
-                return ((File) arg1).delete() ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (action == OPEN_JARX_ACTION)
-            {
-                return new JarFileX((File) arg1);
-            }
-            else if (action == OPEN_JARX_VERIFY_ACTION)
-            {
-                return new JarFileX((File) arg1, ((Boolean) arg2).booleanValue());
-            }
-            else if (action == GET_URL_INPUT_ACTION)
-            {
-                return ((URLConnection) arg1).getInputStream();
-            }
-            else if (action == START_ACTIVATOR_ACTION)
-            {
-                ((BundleActivator) arg1).start((BundleContext) arg2);
-                return null;
-            }
-            else if (action == STOP_ACTIVATOR_ACTION)
-            {
-                ((BundleActivator) arg1).stop((BundleContext) arg2);
-                return null;
-            }
-            else if (action == SYSTEM_EXIT_ACTION)
-            {
-                System.exit(((Integer) arg1).intValue());
-            }
-            else if (action == GET_POLICY_ACTION)
-            {
-                return Policy.getPolicy();
-            }
-            else if (action == CREATE_TMPFILE_ACTION)
-            {
-                return File.createTempFile((String) arg1, (String) arg2,
-                    (File) arg3);
-            }
-            else if (action == OPEN_URLCONNECTION_ACTION)
-            {
-                return ((URL) arg1).openConnection();
-            }
-            else if (action == ADD_EXTENSION_URL)
-            {
-                Method addURL =
-                    URLClassLoader.class.getDeclaredMethod("addURL",
-                    new Class[] {URL.class});
-                addURL.setAccessible(true);
-                addURL.invoke(arg2, new Object[]{arg1});
-            }
-            else if (action == GET_CONSTRUCTOR_ACTION)
-            {
-                return ((Class) arg1).getConstructor((Class[]) arg2);
-            }
-            else if (action == GET_DECLARED_CONSTRUCTOR_ACTION)
-            {
-                return ((Class) arg1).getDeclaredConstructor((Class[]) arg2);
-            }
-            else if (action == GET_METHOD_ACTION)
-            {
-                return ((Class) arg1).getMethod((String) arg2, (Class[]) arg3);
-            }
-            else if (action == INVOKE_METHOD_ACTION)
-            {
-                ((Method) arg1).setAccessible(true);
-                return ((Method) arg1).invoke(arg2, (Object[]) arg3);
-            }
-            else if (action == INVOKE_DIRECTMETHOD_ACTION)
-            {
-                return ((Method) arg1).invoke(arg2, (Object[]) arg3);
-            }
-            else if (action == INVOKE_CONSTRUCTOR_ACTION)
-            {
-                return ((Constructor) arg1).newInstance((Object[]) arg2);
-            }
-            else if (action == SWAP_FIELD_ACTION)
-            {
-                return _swapStaticFieldIfNotClass((Class) arg1,
-                    (Class) arg2, (Class) arg3, (String) arg4);
-            }
-            else if (action == GET_FIELD_ACTION)
-            {
-                Field field = ((Class) arg1).getDeclaredField((String) arg2);
-                field.setAccessible(true);
-                return field.get(arg3);
-            }
-            else if (action == GET_DECLARED_METHOD_ACTION)
-            {
-                return ((Class) arg1).getDeclaredMethod((String) arg2, (Class[]) arg3);
-            }
-            else if (action == SET_ACCESSIBLE_ACTION)
-            {
-                ((AccessibleObject) arg1).setAccessible(true);
-            }
-            else if (action == FLUSH_FIELD_ACTION)
-            {
-                _flush(((Class) arg1), arg2);
-            }
-            else if (action == GET_CLASS_LOADER_ACTION)
-            {
-                return ((Class) arg1).getClassLoader();
+                case INITIALIZE_CONTEXT_ACTION:
+                    return AccessController.getContext();
+                case ADD_EXTENSION_URL_ACTION:
+                    Method addURL =
+                        URLClassLoader.class.getDeclaredMethod("addURL",
+                        new Class[] {URL.class});
+                    addURL.setAccessible(true);
+                    addURL.invoke(arg2, new Object[]{arg1});
+                    return null;
+                case CREATE_TMPFILE_ACTION:
+                    return File.createTempFile((String) arg1, (String) arg2, (File) arg3);
+                case CREATE_URL_ACTION:
+                    return new URL((String) arg1, (String) arg2,
+                        ((Integer) arg3).intValue(), (String) arg4,
+                        (URLStreamHandler) arg5);
+                case CREATE_URL_WITH_CONTEXT_ACTION:
+                    return new URL((URL) arg1, (String) arg2, (URLStreamHandler) arg3);
+                case DELETE_FILE_ACTION:
+                    return ((File) arg1).delete() ? Boolean.TRUE : Boolean.FALSE;
+                case EXEC_ACTION:
+                    return Runtime.getRuntime().exec((String) arg1);
+                case FILE_EXISTS_ACTION:
+                    return ((File) arg1).exists() ? Boolean.TRUE : Boolean.FALSE;
+                case FILE_IS_DIRECTORY_ACTION:
+                    return ((File) arg1).isDirectory() ? Boolean.TRUE : Boolean.FALSE;
+                case FOR_NAME_ACTION:
+                    return Class.forName((String) arg1);
+                case GET_ABSOLUTE_PATH_ACTION:
+                    return ((File) arg1).getAbsolutePath();
+                case GET_CONSTRUCTOR_ACTION:
+                    return ((Class) arg1).getConstructor((Class[]) arg2);
+                case GET_DECLARED_CONSTRUCTOR_ACTION:
+                    return ((Class) arg1).getDeclaredConstructor((Class[]) arg2);
+                case GET_DECLARED_METHOD_ACTION:
+                    return ((Class) arg1).getDeclaredMethod((String) arg2, (Class[]) arg3);
+                case GET_FIELD_ACTION:
+                    Field field = ((Class) arg1).getDeclaredField((String) arg2);
+                    field.setAccessible(true);
+                    return field.get(arg3);
+                case GET_FILE_INPUT_ACTION:
+                    return new FileInputStream((File) arg1);
+                case GET_FILE_OUTPUT_ACTION:
+                    return new FileOutputStream((File) arg1);
+                case TO_URI_ACTION:
+                    return ((File) arg1).toURI();
+                case GET_METHOD_ACTION:
+                    return ((Class) arg1).getMethod((String) arg2, (Class[]) arg3);
+                case GET_POLICY_ACTION:
+                    return Policy.getPolicy();
+                case GET_PROPERTY_ACTION:
+                    return System.getProperty((String) arg1, (String) arg2);
+                case GET_PARENT_CLASS_LOADER_ACTION:
+                    return ((ClassLoader) arg1).getParent();
+                case GET_SYSTEM_CLASS_LOADER_ACTION:
+                    return ClassLoader.getSystemClassLoader();
+                case GET_URL_INPUT_ACTION:
+                    return ((URLConnection) arg1).getInputStream();
+                case INVOKE_CONSTRUCTOR_ACTION:
+                    return ((Constructor) arg1).newInstance((Object[]) arg2);
+                case INVOKE_DIRECTMETHOD_ACTION:
+                    return ((Method) arg1).invoke(arg2, (Object[]) arg3);
+                case INVOKE_METHOD_ACTION:
+                    ((Method) arg1).setAccessible(true);
+                    return ((Method) arg1).invoke(arg2, (Object[]) arg3);
+                case LIST_DIRECTORY_ACTION:
+                    return ((File) arg1).listFiles();
+                case MAKE_DIRECTORIES_ACTION:
+                    return ((File) arg1).mkdirs() ? Boolean.TRUE : Boolean.FALSE;
+                case MAKE_DIRECTORY_ACTION:
+                    return ((File) arg1).mkdir() ? Boolean.TRUE : Boolean.FALSE;
+                case OPEN_ZIPFILE_ACTION:
+                    return new ZipFile((File) arg1);
+                case OPEN_URLCONNECTION_ACTION:
+                    return ((URL) arg1).openConnection();
+                case RENAME_FILE_ACTION:
+                    return ((File) arg1).renameTo((File) arg2) ? Boolean.TRUE : Boolean.FALSE;
+                case SET_ACCESSIBLE_ACTION:
+                    ((AccessibleObject) arg1).setAccessible(true);
+                    return null;
+                case START_ACTIVATOR_ACTION:
+                    ((BundleActivator) arg1).start((BundleContext) arg2);
+                    return null;
+                case STOP_ACTIVATOR_ACTION:
+                    ((BundleActivator) arg1).stop((BundleContext) arg2);
+                    return null;
+                case SWAP_FIELD_ACTION:
+                    return _swapStaticFieldIfNotClass((Class) arg1,
+                        (Class) arg2, (Class) arg3, (String) arg4);
+                case SYSTEM_EXIT_ACTION:
+                    System.exit(((Integer) arg1).intValue());
+                case FLUSH_FIELD_ACTION:
+                    _flush(((Class) arg1), arg2);
+                    return null;
+                case GET_CLASS_LOADER_ACTION:
+                    return ((Class) arg1).getClassLoader();
+                case INVOKE_BUNDLE_FIND_HOOK:
+                    ((org.osgi.framework.hooks.bundle.FindHook) arg1).find(
+                        (BundleContext) arg2, (Collection<Bundle>) arg3);
+                    return null;
+                case INVOKE_BUNDLE_EVENT_HOOK:
+                    ((org.osgi.framework.hooks.bundle.EventHook) arg1).event(
+                        (BundleEvent) arg2, (Collection<BundleContext>) arg3);
+                    return null;
+                case INVOKE_WEAVING_HOOK:
+                    ((org.osgi.framework.hooks.weaving.WeavingHook) arg1).weave(
+                        (org.osgi.framework.hooks.weaving.WovenClass) arg2);
+                    return null;
+                case INVOKE_SERVICE_EVENT_HOOK:
+                    ((org.osgi.framework.hooks.service.EventHook) arg1).event(
+                        (ServiceEvent) arg2, (Collection<BundleContext>) arg3);
+                    return null;
+                case INVOKE_SERVICE_FIND_HOOK:
+                    ((org.osgi.framework.hooks.service.FindHook) arg1).find(
+                        (BundleContext) arg2, (String) arg3, (String) arg4,
+                        ((Boolean) arg5).booleanValue(),
+                        (Collection<ServiceReference<?>>) arg6);
+                    return null;
+                case INVOKE_SERVICE_LISTENER_HOOK_ADDED:
+                    ((org.osgi.framework.hooks.service.ListenerHook) arg1).added(
+                        (Collection<ListenerHook.ListenerInfo>) arg2);
+                    return null;
+                case INVOKE_SERVICE_LISTENER_HOOK_REMOVED:
+                    ((org.osgi.framework.hooks.service.ListenerHook) arg1).removed(
+                        (Collection<ListenerHook.ListenerInfo>) arg2);
+                    return null;
+                case INVOKE_SERVICE_EVENT_LISTENER_HOOK:
+                    ((org.osgi.framework.hooks.service.EventListenerHook) arg1).event(
+                        (ServiceEvent) arg2,
+                        (Map<BundleContext, Collection<ListenerHook.ListenerInfo>>) arg3);
+                    return null;
+                case INVOKE_RESOLVER_HOOK_FACTORY:
+                    return ((org.osgi.framework.hooks.resolver.ResolverHookFactory) arg1).begin(
+                        (Collection<BundleRevision>) arg2);
+                case INVOKE_RESOLVER_HOOK_RESOLVABLE:
+                    ((org.osgi.framework.hooks.resolver.ResolverHook) arg1).filterResolvable(
+                        (Collection<BundleRevision>) arg2);
+                    return null;
+                case INVOKE_RESOLVER_HOOK_SINGLETON:
+                    ((org.osgi.framework.hooks.resolver.ResolverHook) arg1)
+                        .filterSingletonCollisions(
+                            (BundleCapability) arg2,
+                            (Collection<BundleCapability>) arg3);
+                    return null;
+                case INVOKE_RESOLVER_HOOK_MATCHES:
+                    ((org.osgi.framework.hooks.resolver.ResolverHook) arg1).filterMatches(
+                        (BundleRequirement) arg2,
+                        (Collection<BundleCapability>) arg3);
+                    return null;
+                case INVOKE_RESOLVER_HOOK_END:
+                    ((org.osgi.framework.hooks.resolver.ResolverHook) arg1).end();
+                    return null;
             }
 
             return null;
